@@ -1,11 +1,25 @@
 # Paranoid::IO::Lockfile -- Paranoid Lockfile support
 #
-# (c) 2005 - 2017, Arthur Corliss <corliss@digitalmages.com>
+# $Id: lib/Paranoid/IO/Lockfile.pm, 2.09 2021/12/28 15:46:49 acorliss Exp $
 #
-# $Id: lib/Paranoid/IO/Lockfile.pm, 2.05 2017/02/06 01:48:57 acorliss Exp $
+# This software is free software.  Similar to Perl, you can redistribute it
+# and/or modify it under the terms of either:
 #
-#    This software is licensed under the same terms as Perl, itself.
-#    Please see http://dev.perl.org/licenses/ for more information.
+#   a)     the GNU General Public License
+#          <https://www.gnu.org/licenses/gpl-1.0.html> as published by the 
+#          Free Software Foundation <http://www.fsf.org/>; either version 1
+#          <https://www.gnu.org/licenses/gpl-1.0.html>, or any later version
+#          <https://www.gnu.org/licenses/license-list.html#GNUGPL>, or
+#   b)     the Artistic License 2.0
+#          <https://opensource.org/licenses/Artistic-2.0>,
+#
+# subject to the following additional term:  No trademark rights to
+# "Paranoid" have been or are conveyed under any of the above licenses.
+# However, "Paranoid" may be used fairly to describe this unmodified
+# software, in good faith, but not as a trademark.
+#
+# (c) 2005 - 2020, Arthur Corliss (corliss@digitalmages.com)
+# (tm) 2008 - 2020, Paranoid Inc. (www.paranoid.com)
 #
 #####################################################################
 
@@ -28,7 +42,7 @@ use Paranoid;
 use Paranoid::Debug qw(:all);
 use Paranoid::IO;
 
-($VERSION) = ( q$Revision: 2.05 $ =~ /(\d+(?:\.\d+)+)/sm );
+($VERSION) = ( q$Revision: 2.09 $ =~ /(\d+(?:\.\d+)+)/sm );
 
 @EXPORT      = qw(plock pexclock pshlock punlock);
 @EXPORT_OK   = @EXPORT;
@@ -52,19 +66,19 @@ sub plock {
 
     my $filename = shift;
     my $type     = shift;
-    my $mode     = shift;
+    my $perms    = shift;
     my ( $rv, $fh );
 
-    pdebug( 'entering w/(%s)(%s)(%s)', PDLEVEL1, $filename, $type, $mode );
+    pdebug( 'entering w/(%s)(%s)(%s)', PDLEVEL1, $filename, $type, $perms );
     pIn();
 
     # Set the defaults
-    $mode = PRIV_UMASK unless defined $mode;
-    $type = LOCK_EX    unless defined $type;
+    $perms = PRIV_UMASK unless defined $perms;
+    $type  = LOCK_EX    unless defined $type;
 
     # Open the file and apply the lock
-    $fh = popen( $filename, O_RDWR | O_CREAT | O_EXCL, $mode )
-        || popen( $filename, O_RDWR, $mode );
+    $fh = popen( $filename, O_RDWR | O_CREAT | O_EXCL, $perms )
+        || popen( $filename, O_RDWR, $perms );
     $rv = pflock( $filename, $type ) if defined $fh;
 
     pOut();
@@ -149,7 +163,7 @@ Paranoid::IO::Lockfile - Paranoid Lockfile support
 
 =head1 VERSION
 
-$Id: lib/Paranoid/IO/Lockfile.pm, 2.05 2017/02/06 01:48:57 acorliss Exp $
+$Id: lib/Paranoid/IO/Lockfile.pm, 2.09 2021/12/28 15:46:49 acorliss Exp $
 
 =head1 SYNOPSIS
 
@@ -173,6 +187,18 @@ L<Paranoid::IO> functions it removes the small tedium of having to perform
 the multiple opens required to ensure all processes are working off the same
 files while avoiding race conditions.
 
+=head1 IMPORT LISTS
+
+This module exports the following symbols by default:
+
+    plock pexclock pshlock punlock
+
+The following specialized import lists also exist:
+
+    List        Members
+    --------------------------------------------------------
+    all         @defaults
+
 =head1 SUBROUTINES/METHODS
 
 =head2 plock
@@ -242,8 +268,22 @@ Arthur Corliss (corliss@digitalmages.com)
 
 =head1 LICENSE AND COPYRIGHT
 
-This software is licensed under the same terms as Perl, itself. 
-Please see http://dev.perl.org/licenses/ for more information.
+This software is free software.  Similar to Perl, you can redistribute it
+and/or modify it under the terms of either:
 
-(c) 2005 - 2017, Arthur Corliss (corliss@digitalmages.com)
+  a)     the GNU General Public License
+         <https://www.gnu.org/licenses/gpl-1.0.html> as published by the 
+         Free Software Foundation <http://www.fsf.org/>; either version 1
+         <https://www.gnu.org/licenses/gpl-1.0.html>, or any later version
+         <https://www.gnu.org/licenses/license-list.html#GNUGPL>, or
+  b)     the Artistic License 2.0
+         <https://opensource.org/licenses/Artistic-2.0>,
+
+subject to the following additional term:  No trademark rights to
+"Paranoid" have been or are conveyed under any of the above licenses.
+However, "Paranoid" may be used fairly to describe this unmodified
+software, in good faith, but not as a trademark.
+
+(c) 2005 - 2020, Arthur Corliss (corliss@digitalmages.com)
+(tm) 2008 - 2020, Paranoid Inc. (www.paranoid.com)
 
